/* make a sparse file */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/mman.h>

#define USE_MMAP
#ifndef MAP_FAILED
#define MAP_FAILED ((void *)-1)
#endif

int
main(int argc, char *argv[])
{
  int fd, ret;
  char buf[100];
#ifdef USE_MMAP
  caddr_t pa;
#endif /* USE_MMAP */

  if (argc != 2) {
    printf("usage: %s file\n", argv[0]);
    exit(1);
  }

  /*   unlink(argv[1]); */

  fd = open(argv[1], O_RDWR|O_CREAT, 0666);
  if (fd < 0) {
    perror(argv[1]);
    exit(1);
  }
  printf("opened file %s\n", argv[1]);

  /* write first block */
  strcpy(buf, "hello");
  ret = write(fd, buf, strlen(buf));
  if (ret != strlen(buf)) {
    perror("write");
    exit(1);
  }
  printf("wrote first block\n");

  /* seek several blocks ahead */
  ret = lseek(fd, (off_t) 8192*4+2, SEEK_SET);
  if (ret < 0) {
    perror("lseek");
    exit(1);
  }
  printf("lseek past end of file ok\n");

  /* write first block */
  strcpy(buf, "world!");
  ret = write(fd, buf, strlen(buf));
  if (ret != strlen(buf)) {
    perror("write");
    exit(1);
  }
  printf("wrote first block\n");

#ifdef USE_MMAP
  /* now mmap the file and modify a middle byte in it */
  pa = mmap(0,
	     8192*4,
	     PROT_READ|PROT_WRITE,
	     MAP_SHARED,
	     fd,
	     0);
  if (pa == MAP_FAILED) {
    perror("mmap");
    exit(1);
  }
  printf("file is mapped with address 0x%x\n", (int) pa);

  pa[0]++;
  pa[8192*3] = 'z';
  pa[8192*4-1]++;

  printf("modified some page\n");

  if (munmap(pa, 8192*4) < 0) {
    perror("munmap");
    exit(1);
  }
  printf("file is unmapped\n");
#endif /* USE_MMAP */

  close(fd);
  exit(0);
}
